@@ -1,8 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: bufwin.scm,v 1.313 2003/02/14 18:28:11 cph Exp $
-
-Copyright 1986, 1989-2000, 2002 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -228,12 +228,25 @@ USA.
   (with-instance-variables buffer-window window (point*)
     (set! point point*)))
 
-(define-integrable (%set-window-point-index! window index)
+(define (%set-window-point-index! window index)
+  #| Optimization causes lossage.  -- cph
+  (let ((point (%window-point window)))
+    (if point
+	(set-mark-index! point index)
+	(%set-window-point! window
+			    (make-permanent-mark (%window-group window)
+						 index
+						 #t))))
+  |#
   (%set-window-point! window
 		      (make-permanent-mark (%window-group window)
 					   index
 					   #t)))
 
+(define (set-window-point-index! window index)
+  (%set-window-point-index! window index)
+  (%set-buffer-point-index! (%window-buffer window) index))
+
 (define-integrable (%window-cursor-inferior window)
   (with-instance-variables buffer-window window () cursor-inferior))
 
@@ -616,21 +629,21 @@ USA.
 
 (define (%window-line-start-index? window index)
   (or (%window-group-start-index? window index)
-      (char=? (string-ref (group-text (%window-group window))
-			  (fix:- (group-index->position-integrable
-				  (%window-group window)
-				  index
-				  #f)
-				 1))
+      (char=? (xstring-ref (group-text (%window-group window))
+			   (fix:- (group-index->position-integrable
+				   (%window-group window)
+				   index
+				   #f)
+				  1))
 	      #\newline)))
 
 (define (%window-line-end-index? window index)
   (or (%window-group-end-index? window index)
-      (char=? (string-ref (group-text (%window-group window))
-			  (group-index->position-integrable
-			   (%window-group window)
-			   index
-			   #t))
+      (char=? (xstring-ref (group-text (%window-group window))
+			   (group-index->position-integrable
+			    (%window-group window)
+			    index
+			    #t))
 	      #\newline)))
 
 (define (clip-mark-to-display window mark)
@@ -910,9 +923,8 @@ USA.
     (if (%window-debug-trace window)
 	((%window-debug-trace window) 'window window 'set-point! mark))
     (let ((mask (set-interrupt-enables! interrupt-mask/gc-ok)))
-      (%set-window-point-index! window (mark-index mark))
+      (set-window-point-index! window (mark-index mark))
       (%set-window-point-moved?! window 'SINCE-START-SET)
-      (%set-buffer-point! (%window-buffer window) mark)
       (window-needs-redisplay! window)
       (set-interrupt-enables! mask)
       unspecific)))
@@ -988,7 +1000,7 @@ This number is a percentage, where 0 is the window's top and 100 the bottom."
 	       unspecific))
 	    (point-y
 	     (let ((mask (set-interrupt-enables! interrupt-mask/gc-ok)))
-	       (%set-window-point-index!
+	       (set-window-point-index!
 		window
 		(or (predict-index window start y-start
 				   (vector-ref cws 4) point-y)
@@ -1237,7 +1249,7 @@ If this is zero, point is always centered after it moves off screen."
 	(begin
 	  (if point-not-visible-error?
 	      (error "point not visible at end of redisplay"))
-	  (%set-window-point-index!
+	  (set-window-point-index!
 	   window
 	   (or (predict-index window
 			      (%window-start-line-index window)
