@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: evlcom.scm,v 1.71 2005/04/12 18:39:46 cph Exp $
-
-Copyright 1986,1989,1991,1992,1993,1994 Massachusetts Institute of Technology
-Copyright 1995,1997,1998,1999,2000,2001 Massachusetts Institute of Technology
-Copyright 2003,2004,2005 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -20,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -38,13 +36,16 @@ If 'DEFAULT, use the default (REP loop) environment."
   'DEFAULT
   #f
   (lambda (object)
-    (if (or (eq? 'DEFAULT object) (list-of-type? object symbol?))
+    (if (or (eq? 'DEFAULT object)
+	    (list-of-type? object symbol?)
+	    (procedure-of-arity? object 1))
 	object
 	(call-with-current-continuation
 	 (lambda (k)
 	   (bind-condition-handler (list condition-type:error)
 	       (lambda (condition)
 		 condition
+		 (message "Ignoring bad evaluation environment: " object)
 		 (k 'DEFAULT))
 	     (lambda ()
 	       (->environment object))))))))
@@ -59,18 +60,44 @@ If 'DEFAULT, use the default (REP loop) environment."
     (if buffer (normal-buffer-evaluation-mode buffer))))
 
 (define (normal-buffer-evaluation-mode buffer)
-  (let ((environment (ref-variable-object scheme-environment))
-	(evaluate-inferior (ref-variable-object evaluate-in-inferior-repl))
+  (let ((env (ref-variable-object scheme-environment))
+	(inf-repl (ref-variable-object evaluate-in-inferior-repl))
 	(run-light (ref-variable-object run-light)))
-    (if (and (not (repl-buffer? buffer))
-	     (not (variable-local-value? buffer evaluate-inferior))
-	     (and (variable-local-value? buffer environment)
-		  (not (eq? 'DEFAULT
-			    (variable-local-value buffer environment)))))
-	(begin
-	  (define-variable-local-value! buffer evaluate-inferior #f)
-	  (if (not (variable-local-value? buffer run-light))
-	      (define-variable-local-value! buffer run-light #f))))))
+    (if (not (repl-buffer? buffer))
+	(let ((value
+	       (if (variable-local-value? buffer env)
+		   (variable-local-value buffer env)
+		   'DEFAULT)))
+	  (cond ((eq? 'DEFAULT value)
+		 (undefine-variable-local-value! buffer inf-repl)
+		 (undefine-variable-local-value! buffer run-light))
+		((procedure? value)
+		 (define-variable-local-value! buffer inf-repl
+		   (lambda (buffer) buffer (eq? 'DEFAULT (value 'DEFAULT))))
+		 ;; Force run-light to be set:
+		 (evaluate-in-inferior-repl? buffer))
+		(else
+		 (define-variable-local-value! buffer inf-repl #f)
+		 (define-variable-local-value! buffer run-light #f)))))))
+
+(define (evaluate-in-inferior-repl? buffer)
+  (if buffer
+      (let ((buffer (->buffer buffer))
+	    (var (ref-variable-object evaluate-in-inferior-repl)))
+	(let ((value (variable-local-value buffer var)))
+	  (if (procedure? value)
+	      (let ((value (value buffer)))
+		(let ((run-light (ref-variable-object run-light)))
+		  (if value
+		      (undefine-variable-local-value! buffer run-light)
+		      (define-variable-local-value! buffer run-light #f)))
+		(invoke-variable-assignment-daemons! buffer var)
+		value)
+	      value)))
+      (let ((value (ref-variable evaluate-in-inferior-repl #f)))
+	(if (procedure? value)
+	    (value #f)
+	    value))))
 
 (define-variable debug-on-evaluation-error
   "True means enter debugger if an evaluation error is signalled.
@@ -107,7 +134,7 @@ and the output, if non-null, is shown in a pop-up buffer."
 Also, the inferior REPL's run light appears in all Scheme mode buffers.
 Otherwise, expressions are evaluated directly by the commands."
   #t
-  boolean?)
+  (lambda (object) (or (boolean? object) (procedure-of-arity? object 1))))
 
 (define-variable transcript-buffer-name
   "Name of evaluation transcript buffer.
@@ -178,7 +205,7 @@ With an argument, prompts for the evaluation environment."
     (let ((buffer (mark-buffer (region-start region))))
       (cond ((ref-variable disable-evaluation-commands buffer)
 	     (editor-error "Evaluation commands disabled in this buffer."))
-	    ((ref-variable evaluate-in-inferior-repl buffer)
+	    ((evaluate-in-inferior-repl? buffer)
 	     (inferior-repl-eval-region (current-repl-buffer buffer) region))
 	    (else
 	     (evaluate-region region (evaluation-environment buffer #f)))))))
@@ -196,7 +223,7 @@ The values are printed in the typein window."
     (let ((buffer (current-buffer)))
       (cond ((ref-variable disable-evaluation-commands buffer)
 	     (editor-error "Evaluation commands disabled in this buffer."))
-	    ((and (ref-variable evaluate-in-inferior-repl buffer)
+	    ((and (evaluate-in-inferior-repl? buffer)
 		  (current-repl-buffer* buffer))
 	     => (lambda (buffer)
 		  (inferior-repl-eval-expression buffer expression)))
@@ -217,10 +244,9 @@ The values are printed in the typein window."
 Has no effect if evaluate-in-inferior-repl is false."
   ()
   (lambda ()
-    (let ((buffer (current-buffer)))
-      (if (ref-variable evaluate-in-inferior-repl buffer)
-	  ((ref-command inferior-cmdl-abort-top-level))
-	  (editor-error "Nothing to abort.")))))
+    (if (evaluate-in-inferior-repl? (current-buffer))
+	((ref-command inferior-cmdl-abort-top-level))
+	(editor-error "Nothing to abort."))))
 
 (define-command set-environment
   "Make ENVIRONMENT the current evaluation environment."
@@ -278,9 +304,8 @@ Has no effect if evaluate-in-inferior-repl is false."
     (read-from-string
      (apply prompt-for-string
 	    prompt
-	    (if (default-object? default)
-		#f
-		(write-to-string default))
+	    (and (not (default-object? default))
+		 default)
 	    'MODE
 	    (lambda (buffer)
 	      (set-buffer-major-mode! buffer
@@ -348,34 +373,32 @@ Has no effect if evaluate-in-inferior-repl is false."
 		(cons expression (loop)))))))))
 
 (define (evaluation-environment #!optional buffer global-ok?)
-  (let ((buffer
-	 (if (default-object? buffer)
-	     (current-buffer)
-	     (->buffer buffer)))
-	(non-default
-	 (lambda (object)
-	   (if (environment? object)
-	       object
-	       (let ((package (name->package object)))
-		 (cond (package
-			(package/environment package))
-		       ((if (default-object? global-ok?) #t global-ok?)
-			system-global-environment)
-		       (else
-			(editor-error "Package not loaded: " object))))))))
+  (let ((buffer (->buffer buffer)))
+    (evaluation-environment-no-repl
+     buffer
+     (let ((repl-buffer
+	    (and (evaluate-in-inferior-repl? buffer)
+		 (current-repl-buffer* buffer))))
+       (if (and repl-buffer
+		(not (eq? repl-buffer buffer)))
+	   (evaluation-environment-no-repl repl-buffer)
+	   #!default))
+     global-ok?)))
+
+(define (evaluation-environment-no-repl #!optional buffer default global-ok?)
+  (let ((buffer (->buffer buffer))
+	(default
+	  (if (default-object? default)
+	      (nearest-repl/environment)
+	      default))
+	(global-ok? (if (default-object? global-ok?) #t global-ok?)))
     (let ((environment (ref-variable scheme-environment buffer)))
-      (if (eq? 'DEFAULT environment)
-	  (let ((repl-buffer
-		 (and (ref-variable evaluate-in-inferior-repl buffer)
-		      (current-repl-buffer* buffer))))
-	    (if repl-buffer
-		(let ((environment
-		       (ref-variable scheme-environment repl-buffer)))
-		  (if (eq? 'DEFAULT environment)
-		      (nearest-repl/environment)
-		      (non-default environment)))
-		(nearest-repl/environment)))
-	  (non-default environment)))))
+      (cond ((eq? 'DEFAULT environment) default)
+	    ((environment? environment) environment)
+	    ((procedure? environment) (environment default))
+	    ((name->package environment) => package/environment)
+	    (global-ok? system-global-environment)
+	    (else (editor-error "Package not loaded: " environment))))))
 
 (define-variable run-light
   "Scheme run light.  Not intended to be modified by users.
