@@ -1,9 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: intmod.scm,v 1.119 2003/02/14 18:28:12 cph Exp $
-
-Copyright 1986,1989,1991,1992,1993,1999 Massachusetts Institute of Technology
-Copyright 2000,2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -157,17 +156,13 @@ evaluated in the specified inferior REPL buffer."
   unspecific)
 
 (define (current-repl-buffer #!optional buffer)
-  (let ((buffer
-	 (current-repl-buffer* (if (default-object? buffer) #f buffer))))
-    (if (not buffer)
+  (let ((repl-buffer (current-repl-buffer* buffer)))
+    (if (not repl-buffer)
 	(error "No REPL to evaluate in."))
-    buffer))
+    repl-buffer))
 
 (define (current-repl-buffer* #!optional buffer)
-  (let ((buffer
-	 (if (or (default-object? buffer) (not buffer))
-	     (current-buffer)
-	     buffer)))
+  (let ((buffer (->buffer buffer)))
     (if (repl-buffer? buffer)
 	buffer
 	(or (local-repl-buffer buffer)
@@ -201,7 +196,7 @@ evaluated in the specified inferior REPL buffer."
 	 (car buffers))))
 
 (define (repl-buffer-list)
-  (set! repl-buffers (list-transform-positive repl-buffers buffer-alive?))
+  (set! repl-buffers (filter! buffer-alive? repl-buffers))
   repl-buffers)
 
 (define (repl-buffer? buffer)
@@ -260,11 +255,26 @@ evaluated in the specified inferior REPL buffer."
 		(without-interrupts
 		 (lambda ()
 		   (set-car! (buffer-modes buffer) mode)
-		   (set-buffer-comtabs! buffer (mode-comtabs mode))))
+		   (switch-comtabs! buffer mode mode*)))
 		(buffer-modeline-event! buffer 'BUFFER-MODES))
 	      (begin
 		(set-buffer-major-mode! buffer mode)
 		(attach-buffer-interface-port! buffer port)))))))
+
+(define (switch-comtabs! buffer new-mode old-mode)
+  (let ((comtabs (buffer-comtabs buffer))
+	(new-comtabs (mode-comtabs new-mode))
+	(old-comtabs (mode-comtabs old-mode)))
+    (if (eq? comtabs old-comtabs)
+	(set-buffer-comtabs! buffer new-comtabs)
+	(let loop ((previous comtabs))
+	  (let ((comtabs (cdr previous)))
+	    (cond ((eq? comtabs old-comtabs)
+		   (set-cdr! previous new-comtabs))
+		  ((not (pair? comtabs))
+		   (warn ";Buffer's comtabs do not match its mode:" buffer))
+		  (else
+		   (loop comtabs))))))))
 
 (define (attach-buffer-interface-port! buffer port)
   (if (not (memq buffer repl-buffers))
@@ -320,7 +330,7 @@ evaluated in the specified inferior REPL buffer."
 	      (buffer-list))))
 
 (define (global-run-light-buffer)
-  (and (variable-default-value (ref-variable-object evaluate-in-inferior-repl))
+  (and (evaluate-in-inferior-repl? #f)
        (global-repl-buffer)))
 
 (define (set-global-run-light! value)
@@ -336,12 +346,13 @@ evaluated in the specified inferior REPL buffer."
 
 (add-variable-assignment-daemon!
  (ref-variable-object evaluate-in-inferior-repl)
- (lambda (buffer variable)
-   buffer variable
+ (lambda (buffer variable) buffer variable (reset-run-light!)))
+
+(define (reset-run-light!)
+  (set-global-run-light!
    (let ((buffer (global-run-light-buffer)))
-     (if buffer
-	 (set-global-run-light! (local-run-light buffer))
-	 (set-global-run-light! #f)))))
+     (and buffer
+	  (local-run-light buffer)))))
 
 (define (error-decision repl condition)
   (let ((port (cmdl/port repl)))
@@ -443,7 +454,9 @@ The REPL may be controlled by the following commands:
 \\[inferior-cmdl-abort-top-level] aborts evaluation, returns to top level.
 \\[inferior-cmdl-abort-nearest] aborts evaluation, returns to current level.
 \\[inferior-cmdl-abort-previous] aborts evaluation, goes up one level.
-\\[inferior-cmdl-breakpoint] interrupts evaluation, enters a breakpoint."
+\\[inferior-cmdl-breakpoint] interrupts evaluation, enters a breakpoint.
+
+\\{inferior-repl}"
   (lambda (buffer)
     (event-distributor/invoke! (ref-variable inferior-repl-mode-hook buffer)
 			       buffer)))
@@ -669,24 +682,7 @@ If this is an error, the debugger examines the error condition."
 			 (region-end region)
 			 mark))))
   (let ((port (buffer-interface-port buffer #t)))
-    (let ((input-end
-	   (let ((mark
-		  (let ((end (buffer-end buffer))
-			(end* (region-end region)))
-		    (if (mark~ end end*)
-			(begin
-			  (set-buffer-point! buffer end*)
-			  end*)
-			end))))
-	     (cond ((eqv? #\newline (extract-right-char mark))
-		    (mark1+ mark))
-		   ((line-start? mark)
-		    mark)
-		   (else
-		    (let ((mark (mark-left-inserting-copy mark)))
-		      (insert-newline mark)
-		      (mark-temporary! mark)
-		      mark))))))
+    (let ((input-end (inferior-repl-input-end buffer region)))
       (move-mark-to! (port/mark port) input-end)
       (move-mark-to! (ref-variable comint-last-input-end buffer) input-end))
     (let ((queue (port/expression-queue port)))
@@ -704,6 +700,27 @@ If this is an error, the debugger examines the error condition."
       (if (not (queue-empty? queue))
 	  (end-input-wait port)))))
 
+(define (inferior-repl-input-end buffer region)
+  (receive (mark in-buffer?)
+      (let ((end (buffer-end buffer))
+	    (end* (region-end region)))
+	(if (mark~ end end*)
+	    (values end* #t)
+	    (values end #f)))
+    (let ((mark
+	   (cond ((eqv? #\newline (extract-right-char mark))
+		  (mark1+ mark))
+		 ((line-start? mark)
+		  mark)
+		 (else
+		  (let ((mark (mark-left-inserting-copy mark)))
+		    (insert-newline mark)
+		    (mark-temporary! mark)
+		    mark)))))
+      (if in-buffer?
+	  (set-buffer-point! buffer mark))
+      mark)))
+
 (define (inferior-repl-eval-expression buffer expression)
   (inferior-repl-eval-ok? buffer)
   (call-with-transcript-output-mark buffer
@@ -786,12 +803,6 @@ If this is an error, the debugger examines the error condition."
 		   (make-interface-port-state
 		    thread
 		    (mark-right-inserting-copy (buffer-end buffer))
-		    (make-ring (ref-variable comint-input-ring-size))
-		    (make-queue)
-		    #f
-		    #f
-		    (make-queue)
-		    '()
 		    (register-inferior-thread!
 		     thread
 		     (lambda () (process-output-queue port)))))))
@@ -801,16 +812,20 @@ If this is an error, the debugger examines the error condition."
   (and (port? object)
        (interface-port-state? (port/state object))))
 
-(define-structure (interface-port-state (conc-name interface-port-state/))
+(define-structure (interface-port-state
+		   (conc-name interface-port-state/)
+		   (constructor make-interface-port-state
+				(thread mark output-registration)))
   (thread #f read-only #t)
   (mark #f read-only #t)
-  (input-ring #f read-only #t)
-  (expression-queue #f read-only #t)
-  current-queue-element
-  command-char
-  (output-queue #f read-only #t)
-  output-strings
-  (output-registration #f read-only #t))
+  (input-ring (make-ring (ref-variable comint-input-ring-size)) read-only #t)
+  (expression-queue (make-queue) read-only #t)
+  (current-queue-element #f)
+  (command-char #f)
+  (output-queue (make-queue) read-only #t)
+  (output-strings '())
+  (output-registration #f read-only #t)
+  (bytes-written 0))
 
 (define-integrable (port/thread port)
   (interface-port-state/thread (port/state port)))
@@ -850,19 +865,26 @@ If this is an error, the debugger examines the error condition."
 
 (define-integrable (port/output-registration port)
   (interface-port-state/output-registration (port/state port)))
+
+(define-integrable (port/bytes-written port)
+  (interface-port-state/bytes-written (port/state port)))
+
+(define-integrable (set-port/bytes-written! port n)
+  (set-interface-port-state/bytes-written! (port/state port) n))
 
 ;;; Output operations
 
 (define (operation/write-char port char)
-  (enqueue-output-string! port (string char)))
+  (guarantee-8-bit-char char)
+  (enqueue-output-string! port (string char))
+  1)
 
 (define (operation/write-substring port string start end)
-  (enqueue-output-string! port (substring string start end)))
-
-(define (operation/fresh-line port)
-  (enqueue-output-operation!
-   port
-   (lambda (mark transcript?) transcript? (guarantee-newline mark) #t)))
+  (if (string? string)
+      (begin
+	(enqueue-output-string! port (substring string start end))
+	(fix:- end start))
+      (generic-port-operation:write-substring port string start end)))
 
 (define (operation/beep port)
   (enqueue-output-operation!
@@ -876,7 +898,7 @@ If this is an error, the debugger examines the error condition."
 	   (and (not (null? windows))
 		(apply min (map window-x-size windows)))))))
 
-(define (operation/write-result port expression value hash-number)
+(define (operation/write-result port expression value hash-number environment)
   (let ((buffer (port/buffer port))
 	(other-buffer?
 	 (memq (operation/current-expression-context port expression)
@@ -887,7 +909,7 @@ If this is an error, the debugger examines the error condition."
 			  (and (ref-variable enable-transcript-buffer buffer)
 			       (transcript-buffer)))
 	(begin
-	  (default/write-result port expression value hash-number)
+	  (default/write-result port expression value hash-number environment)
 	  (if (and other-buffer? (not (mark-visible? (port/mark port))))
 	      (transcript-write value #f))))))
 
@@ -899,10 +921,18 @@ If this is an error, the debugger examines the error condition."
 (define (enqueue-output-string! port string)
   (let ((interrupt-mask (set-interrupt-enables! interrupt-mask/gc-ok)))
     (set-port/output-strings! port (cons string (port/output-strings port)))
+    (set-port/bytes-written! port
+			     (+ (port/bytes-written port)
+				(string-length string)))
     (inferior-thread-output!/unsafe (port/output-registration port))
     (set-interrupt-enables! interrupt-mask)
     unspecific))
 
+;;; We assume here that none of the OPERATORs passed to this procedure
+;;; generate any output in the REPL buffer, and consequently we don't
+;;; need to update bytes-written here.  Review of the current usage of
+;;; this procedure confirms the assumption. 
+
 (define (enqueue-output-operation! port operator)
   (let ((interrupt-mask (set-interrupt-enables! interrupt-mask/gc-ok)))
     (let ((strings (port/output-strings port)))
@@ -958,9 +988,6 @@ If this is an error, the debugger examines the error condition."
 
 ;;; Input operations
 
-(define (operation/peek-char port)
-  (error "PEEK-CHAR not supported on this port:" port))
-
 (define (operation/read-char port)
   (error "READ-CHAR not supported on this port:" port))
 
@@ -1018,8 +1045,11 @@ If this is an error, the debugger examines the error condition."
 
 ;;; Prompting
 
-(define (operation/prompt-for-expression port prompt)
-  (unsolicited-prompt port prompt-for-expression prompt))
+(define (operation/prompt-for-expression port environment prompt)
+  (unsolicited-prompt port
+		      (lambda (prompt)
+			(prompt-for-expression prompt #!default environment))
+		      prompt))
 
 (define (operation/prompt-for-confirmation port prompt)
   (unsolicited-prompt port prompt-for-confirmation? prompt))
@@ -1062,7 +1092,7 @@ If this is an error, the debugger examines the error condition."
 	  (cond ((eq? value wait-value) (suspend-current-thread) (loop))
 		((eq? value abort-value) (abort->nearest))
 		(else value)))))))
-
+
 (define (when-buffer-selected buffer thunk)
   (if (current-buffer? buffer)
       (thunk)
@@ -1073,7 +1103,8 @@ If this is an error, the debugger examines the error condition."
 			     (remove-select-buffer-hook buffer hook))))))
 	(add-select-buffer-hook buffer hook))))
 
-(define (operation/prompt-for-command-expression port prompt level)
+(define (operation/prompt-for-command-expression port environment prompt level)
+  environment
   (parse-command-prompt port prompt)
   (read-expression port level))
 
@@ -1120,9 +1151,9 @@ If this is an error, the debugger examines the error condition."
   (make-port-type
    `((WRITE-CHAR ,operation/write-char)
      (WRITE-SUBSTRING ,operation/write-substring)
-     (FRESH-LINE ,operation/fresh-line)
      (BEEP ,operation/beep)
      (X-SIZE ,operation/x-size)
+     (BYTES-WRITTEN ,port/bytes-written)
      (DEBUGGER-FAILURE ,operation/debugger-failure)
      (DEBUGGER-MESSAGE ,operation/debugger-message)
      (DEBUGGER-PRESENTATION ,operation/debugger-presentation)
@@ -1132,7 +1163,6 @@ If this is an error, the debugger examines the error condition."
      (PROMPT-FOR-COMMAND-CHAR ,operation/prompt-for-command-char)
      (SET-DEFAULT-DIRECTORY ,operation/set-default-directory)
      (SET-DEFAULT-ENVIRONMENT ,operation/set-default-environment)
-     (PEEK-CHAR ,operation/peek-char)
      (READ-CHAR ,operation/read-char)
      (READ ,operation/read)
      (CURRENT-EXPRESSION-CONTEXT ,operation/current-expression-context)
