@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: unix.scm,v 1.120 2006/05/15 02:43:54 cph Exp $
-
-Copyright 1989,1991,1992,1993,1994,1995 Massachusetts Institute of Technology
-Copyright 1996,1997,1999,2000,2002,2003 Massachusetts Institute of Technology
-Copyright 2004,2006 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -20,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -323,7 +321,7 @@ Includes the new backup.  Must be > 0."
 	   (string-suffix? extension filename)))))
 
 (define (os/completion-ignored-extensions)
-  (append (list ".bin" ".com" ".ext"
+  (append (list ".bin" ".com" ".ext" ".so"
 		".inf" ".bif" ".bsm" ".bci" ".bcs"
 		".psb" ".moc" ".fni"
 		".bco" ".bld" ".bad" ".glo" ".fre"
@@ -666,8 +664,9 @@ option, instead taking -P <filename>."
   (cons "~/mbox" system-mailboxes))
 
 (define (os/sendmail-program)
-  (or (list-search-positive
-	  '("/usr/lib/sendmail" "/usr/sbin/sendmail" "/usr/ucblib/sendmail")
+  (or (os/find-program "sendmail" #f (ref-variable exec-path) #f)
+      (find-matching-item
+	  '("/usr/sbin/sendmail" "/usr/lib/sendmail" "/usr/ucblib/sendmail")
 	file-executable?)
       "fakemail"))
 
