@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: lambda.scm,v 14.22 2008/01/30 20:02:32 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -250,11 +248,12 @@ USA.
 				       (make-unassigned auxiliary))))
 		(list (let ((environment (make-the-environment)))
 			(make-combination
-			 system-subvector->list
+			 (ucode-primitive system-subvector-to-list)
 			 (list environment
 			       (+ (length required) 3)
-			       (make-combination system-vector-length
-						 (list environment)))))))))
+			       (make-combination
+				(ucode-primitive system-vector-size)
+				(list environment)))))))))
 
 (define (clexpr-components clexpr receiver)
   (slexpr-components clexpr
