@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: parse.scm,v 14.63 2006/06/21 13:45:52 cph Exp $
-
-Copyright 1986,1987,1988,1989,1990,1991 Massachusetts Institute of Technology
-Copyright 1992,1993,1994,1997,1998,1999 Massachusetts Institute of Technology
-Copyright 2001,2002,2003,2004,2005,2006 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -20,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -28,7 +26,9 @@ USA.
 ;;;; Scheme Parser
 ;;; package: (runtime parser)
 
-(declare (usual-integrations))
+(declare (usual-integrations)
+	 (integrate-external "input")
+	 (integrate-external "port"))
 
 (define *parser-radix* 10)
 (define *parser-canonicalize-symbols?* #t)
@@ -73,7 +73,7 @@ USA.
   (let ((handlers (parser-table/initial (db-parser-table db))))
     (let loop ()
       (let* ((position (current-position port db))
-	     (char (read-char port)))
+	     (char (%read-char port db)))
 	(if (eof-object? char)
 	    char
 	    (let ((object ((get-handler char handlers) port db ctx char)))
@@ -87,7 +87,7 @@ USA.
   (list 'CONTINUE-PARSING))
 
 (define (handler:special port db ctx char1)
-  (let ((char2 (read-char/no-eof port)))
+  (let ((char2 (%read-char/no-eof port db)))
     ((get-handler char2 (parser-table/special (db-parser-table db)))
      port db ctx char1 char2)))
 
@@ -185,7 +185,7 @@ USA.
 (define (handler:comment port db ctx char)
   db ctx char
   (let loop ()
-    (let ((char (read-char port)))
+    (let ((char (%read-char port db)))
       (cond ((eof-object? char) char)
 	    ((char=? char #\newline) unspecific)
 	    (else (loop)))))
@@ -194,16 +194,16 @@ USA.
 (define (handler:multi-line-comment port db ctx char1 char2)
   db ctx char1 char2
   (let loop ()
-    (case (read-char/no-eof port)
+    (case (%read-char/no-eof port db)
       ((#\#)
        (let sharp ()
-	 (case (read-char/no-eof port)
+	 (case (%read-char/no-eof port db)
 	   ((#\#) (sharp))
 	   ((#\|) (loop) (loop))
 	   (else (loop)))))
       ((#\|)
        (let vbar ()
-	 (case (read-char/no-eof port)
+	 (case (%read-char/no-eof port db)
 	   ((#\#) unspecific)
 	   ((#\|) (vbar))
 	   (else (loop)))))
@@ -248,29 +248,33 @@ USA.
 
 (define (parse-atom-1 port db prefix quoting?)
   (let ((port* (open-output-string))
-	(canon
+	(table
 	 (if (db-canonicalize-symbols? db)
-	     char-downcase
-	     identity-procedure))
-	(%read
-	 (lambda ()
-	     (if (pair? prefix)
-		 (let ((char (car prefix)))
-		   (set! prefix (cdr prefix))
-		   char)
-		 (read-char/no-eof port))))
-	(%peek
-	 (lambda ()
-	   (if (pair? prefix)
-	       (car prefix)
-	       (peek-char port))))
-	(%discard
-	 (lambda ()
-	   (if (pair? prefix)
-	       (begin
-		 (set! prefix (cdr prefix))
-		 unspecific)
-	       (discard-char port)))))
+	     downcase-table
+	     identity-table)))
+    (define (%canon char)
+      ;; Assumption: No character involved in I/O has bucky bits, and
+      ;; case conversion applies only to ISO-8859-1 characters.
+      (let ((integer (char->integer char)))
+	(if (fix:< integer #x100)
+	    (integer->char (vector-8b-ref table integer))
+	    char)))
+    (define (%read)
+      (if (pair? prefix)
+	  (let ((char (car prefix)))
+	    (set! prefix (cdr prefix))
+	    char)
+	  (%read-char/no-eof port db)))
+    (define (%peek)
+      (if (pair? prefix)
+	  (car prefix)
+	  (%peek-char port db)))
+    (define (%discard)
+      (if (pair? prefix)
+	  (begin
+	    (set! prefix (cdr prefix))
+	    unspecific)
+	  (%read-char port db)))
     (let read-unquoted ((quoted? #f))
       (let ((char (%peek)))
 	(if (or (eof-object? char)
@@ -288,7 +292,7 @@ USA.
 			     (if (char=? char #\|)
 				 (read-unquoted #t)
 				 (begin
-				   (write-char (if (char=? char #\\)
+				   (%write-char (if (char=? char #\\)
 						   (%read)
 						   char)
 					       port*)
@@ -297,11 +301,11 @@ USA.
 		    ((char=? char #\\)
 		     (if quoting?
 			 (begin
-			   (write-char (%read) port*)
+			   (%write-char (%read) port*)
 			   (read-unquoted #t))
 			 (error:illegal-char char)))
 		    (else
-		     (write-char (canon char) port*)
+		     (%write-char (%canon char) port*)
 		     (read-unquoted quoted?)))))))))
 
 (define (handler:list port db ctx char)
@@ -343,12 +347,12 @@ USA.
 	      ignore-extra-list-closes)
 	 continue-parsing)
 	(else
-	 (error:illegal-char char))))
+	 (error:unbalanced-close char))))
 
 (define (handler:close-bracket port db ctx char)
   port db
   (if (not (eq? ctx 'CLOSE-BRACKET-OK))
-      (error:illegal-char char))
+      (error:unbalanced-close char))
   close-bracket)
 
 (define close-parenthesis (list 'CLOSE-PARENTHESIS))
@@ -418,9 +422,9 @@ USA.
 
 (define (handler:unquote port db ctx char)
   ctx char
-  (if (char=? (peek-char/no-eof port) #\@)
+  (if (char=? (%peek-char/no-eof port db) #\@)
       (begin
-	(discard-char port)
+	(%read-char port db)
 	(list 'UNQUOTE-SPLICING (read-object port db)))
       (list 'UNQUOTE (read-object port db))))
 
@@ -429,13 +433,13 @@ USA.
   (call-with-output-string
     (lambda (port*)
       (let loop ()
-	(let ((char (read-char/no-eof port)))
+	(let ((char (%read-char/no-eof port db)))
 	  (case char
 	    ((#\")
 	     unspecific)
 	    ((#\\)
 	     (let ((char
-		    (let ((char (read-char/no-eof port)))
+		    (let ((char (%read-char/no-eof port db)))
 		      (cond ((char-ci=? char #\n) #\newline)
 			    ((char-ci=? char #\t) #\tab)
 			    ((char-ci=? char #\v) #\vt)
@@ -443,23 +447,23 @@ USA.
 			    ((char-ci=? char #\r) #\return)
 			    ((char-ci=? char #\f) #\page)
 			    ((char-ci=? char #\a) #\bel)
-			    ((char->digit char 8) (octal->char char port))
+			    ((char->digit char 8) (octal->char char port db))
 			    (else char)))))
-	       (write-char char port*)
+	       (%write-char char port*)
 	       (loop)))
 	    (else
-	     (write-char char port*)
+	     (%write-char char port*)
 	     (loop))))))))
 
-(define (octal->char c1 port)
+(define (octal->char c1 port db)
   (let ((d1 (char->digit c1 8)))
     (if (or (not d1) (fix:> d1 3))
 	(error:illegal-char c1))
-    (let* ((c2 (read-char/no-eof port))
+    (let* ((c2 (%read-char/no-eof port db))
 	   (d2 (char->digit c2 8)))
       (if (not d2)
 	  (error:illegal-char c2))
-      (let* ((c3 (read-char/no-eof port))
+      (let* ((c3 (%read-char/no-eof port db))
 	     (d3 (char->digit c3 8)))
 	(if (not d3)
 	    (error:illegal-char c3))
@@ -497,10 +501,10 @@ USA.
 
 (define (handler:char port db ctx char1 char2)
   db ctx char1 char2
-  (let ((char (read-char/no-eof port))
+  (let ((char (%read-char/no-eof port db))
 	(at-end?
 	 (lambda ()
-	   (let ((char (peek-char port)))
+	   (let ((char (%peek-char port db)))
 	     (or (eof-object? char)
 		 (atom-delimiter? char))))))
     (if (or (atom-delimiter? char)
@@ -509,11 +513,11 @@ USA.
 	(name->char
 	 (call-with-output-string
 	   (lambda (port*)
-	     (write-char char port*)
+	     (%write-char char port*)
 	     (let loop ()
-	       (write-char (let ((char (read-char/no-eof port)))
+	       (%write-char (let ((char (%read-char/no-eof port db)))
 			     (if (char=? char #\\)
-				 (read-char/no-eof port)
+				 (%read-char/no-eof port db)
 				 char))
 			   port*)
 	       (if (not (at-end?))
@@ -525,24 +529,19 @@ USA.
     (cond ((string-ci=? name "null") '())
 	  ((string-ci=? name "false") #f)
 	  ((string-ci=? name "true") #t)
-	  ((string-ci=? name "optional") lambda-optional-tag)
-	  ((string-ci=? name "rest") lambda-rest-tag)
-	  ((string-ci=? name "key") lambda-key-tag)
-	  ((string-ci=? name "aux") lambda-aux-tag)
-	  ((string-ci=? name "eof") (make-eof-object #f))
+	  ((string-ci=? name "optional") lambda-tag:optional)
+	  ((string-ci=? name "rest") lambda-tag:rest)
+	  ((string-ci=? name "key") lambda-tag:key)
+	  ((string-ci=? name "aux") lambda-tag:aux)
+	  ((string-ci=? name "eof") (eof-object))
 	  ((string-ci=? name "default") (default-object))
 	  ((string-ci=? name "unspecific") unspecific)
 	  (else (error:illegal-named-constant name)))))
 
-(define lambda-optional-tag (object-new-type (ucode-type constant) 3))
-(define lambda-rest-tag (object-new-type (ucode-type constant) 4))
-(define lambda-aux-tag (object-new-type (ucode-type constant) 8))
-(define lambda-key-tag (object-new-type (ucode-type constant) 5))
-
 (define (handler:special-arg port db ctx char1 char2)
   ctx char1
   (let loop ((n (char->digit char2 10)))
-    (let ((char (read-char/no-eof port)))
+    (let ((char (%read-char/no-eof port db)))
       (cond ((char-numeric? char)
 	     (loop (+ (* 10 n) (char->digit char 10))))
 	    ((char=? char #\=)
@@ -573,37 +572,42 @@ USA.
 (define non-shared-object
   (list 'NON-SHARED-OBJECT))
 
-(define (read-char port)
-  (let loop ()
-    (or (input-port/read-char port)
-	(loop))))
+(define (%read-char port db)
+  (let ((char
+	 (let loop ()
+	   (or (input-port/%read-char port)
+	       (loop))))
+	(op (db-discretionary-write-char db)))
+    (if op
+	(op char port))
+    char))
 
-(define (read-char/no-eof port)
-  (let ((char (read-char port)))
+(define (%read-char/no-eof port db)
+  (let ((char (%read-char port db)))
     (if (eof-object? char)
 	(error:premature-eof port))
     char))
 
-(define (discard-char port)
+(define (%peek-char port db)
+  db					;ignore
   (let loop ()
-    (if (not (input-port/discard-char port))
+    (or (input-port/%peek-char port)
 	(loop))))
 
-(define (peek-char port)
-  (let loop ()
-    (or (input-port/peek-char port)
-	(loop))))
-
-(define (peek-char/no-eof port)
-  (let ((char (peek-char port)))
+(define (%peek-char/no-eof port db)
+  (let ((char (%peek-char port db)))
     (if (eof-object? char)
 	(error:premature-eof port))
     char))
 
 (define-structure db
-  (environment #f read-only #t)
+  (radix #f read-only #t)
+  (canonicalize-symbols? #f read-only #t)
+  (associate-positions? #f read-only #t)
+  (parser-table #f read-only #t)
   (shared-objects #f read-only #t)
   (get-position #f read-only #t)
+  (discretionary-write-char #f read-only #t)
   position-mapping)
 
 (define (initial-db port environment)
@@ -614,23 +618,15 @@ USA.
 	     (begin
 	       (guarantee-environment environment #f)
 	       environment))))
-    (make-db environment
+    (make-db (environment-lookup environment '*PARSER-RADIX*)
+	     (environment-lookup environment '*PARSER-CANONICALIZE-SYMBOLS?*)
+	     (environment-lookup environment '*PARSER-ASSOCIATE-POSITIONS?*)
+	     (environment-lookup environment '*PARSER-TABLE*)
 	     (make-shared-objects)
 	     (position-operation port environment)
+	     (port/operation port 'DISCRETIONARY-WRITE-CHAR)
 	     '())))
 
-(define (db-radix db)
-  (environment-lookup (db-environment db) '*PARSER-RADIX*))
-
-(define (db-canonicalize-symbols? db)
-  (environment-lookup (db-environment db) '*PARSER-CANONICALIZE-SYMBOLS?*))
-
-(define (db-associate-positions? db)
-  (environment-lookup (db-environment db) '*PARSER-ASSOCIATE-POSITIONS?*))
-
-(define (db-parser-table db)
-  (environment-lookup (db-environment db) '*PARSER-TABLE*))
-
 (define (position-operation port environment)
   (let ((default (lambda (port) port #f)))
     (if (environment-lookup environment '*PARSER-ASSOCIATE-POSITIONS?*)
@@ -691,6 +687,7 @@ USA.
 (define condition-type:premature-eof)
 (define condition-type:re-shared-object)
 (define condition-type:non-shared-object)
+(define condition-type:unbalanced-close)
 (define error:illegal-bit-string)
 (define error:illegal-boolean)
 (define error:illegal-char)
@@ -704,6 +701,7 @@ USA.
 (define error:premature-eof)
 (define error:re-shared-object)
 (define error:non-shared-object)
+(define error:unbalanced-close)
 
 (define (initialize-condition-types!)
   (set! condition-type:parse-error
@@ -717,7 +715,7 @@ USA.
       (write-string string port)))
   (define-parse-error (illegal-boolean string)
     (lambda (string port)
-      (write-string "Ill-formed boolean: #" port)
+      (write-string "Ill-formed boolean: " port)
       (write-string string port)))
   (define-parse-error (illegal-char char)
     (lambda (char port)
@@ -773,4 +771,8 @@ USA.
       (write-string "Reference to non-shared object: #" port)
       (write n port)
       (write-string "#" port)))
+  (define-parse-error (unbalanced-close char)
+    (lambda (char port)
+      (write-string "Unbalanced close parenthesis: " port)
+      (write char port)))
   unspecific)
\ No newline at end of file
