@@ -1,8 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: pathnm.scm,v 14.36 2003/02/14 18:28:33 cph Exp $
-
-Copyright (c) 1988-2001 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -98,15 +98,13 @@ these rules:
 
 (define-structure (pathname
 		   (type vector)
-		   (named ((ucode-primitive string->symbol)
-			   "#[(runtime pathname)pathname]"))
+		   (named '|#[(runtime pathname)pathname]|)
 		   (constructor %make-pathname)
 		   (conc-name %pathname-)
 		   (print-procedure
-		    (standard-unparser-method 'PATHNAME
-		      (lambda (pathname port)
-			(write-char #\space port)
-			(write (->namestring pathname) port)))))
+		    (simple-unparser-method 'PATHNAME
+		      (lambda (pathname)
+			(list (->namestring pathname))))))
   (host #f read-only #t)
   (device #f read-only #t)
   (directory #f read-only #t)
@@ -114,6 +112,14 @@ these rules:
   (type #f read-only #t)
   (version #f read-only #t))
 
+(define-guarantee pathname "pathname")
+
+(define pathname-parser-method
+  (simple-parser-method
+   (lambda (objects)
+     (and (pair? objects)
+	  (->pathname (car objects))))))
+
 (define (->pathname object)
   (pathname-arg object #f '->PATHNAME))
 
@@ -144,12 +150,6 @@ these rules:
 
 (define (pathname-version pathname)
   (%pathname-version (->pathname pathname)))
-
-(define (pathname-end-of-line-string pathname)
-  (let ((pathname (->pathname pathname)))
-    ((host-type/operation/end-of-line-string
-      (host/type (%pathname-host pathname)))
-     pathname)))
 
 (define (pathname=? x y)
   (let ((x (->pathname x))
@@ -166,6 +166,11 @@ these rules:
     (and (pair? directory)
 	 (eq? (car directory) 'ABSOLUTE))))
 
+(define (pathname-relative? pathname)
+  (let ((directory (pathname-directory pathname)))
+    (and (pair? directory)
+	 (eq? (car directory) 'RELATIVE))))
+
 (define (pathname-wild? pathname)
   (let ((pathname (->pathname pathname)))
     ((host-type/operation/pathname-wild?
@@ -292,6 +297,83 @@ these rules:
 		    (or (%pathname-type pathname) type)
 		    (or (%pathname-version pathname) version))))
 
+(define (pathname->uri pathname)
+  (let ((pathname (->pathname pathname)))
+    (make-uri (if (pathname-absolute? pathname) 'file #f)
+	      #f
+	      (map (lambda (x)
+		     (cond ((eq? x 'WILD) "*")
+			   ((eq? x 'UP) "..")
+			   (else (string->utf8-string x))))
+		   (append (if (pathname-absolute? pathname)
+			       (list "")
+			       '())
+			   (let ((device (pathname-device pathname))
+				 (directory (pathname-directory pathname)))
+			     (if (missing-component? device)
+				 (if (missing-component? directory)
+				     '()
+				     (cdr directory))
+				 (cons device (cdr directory))))
+			   (let ((name (file-namestring pathname)))
+			     (if (missing-component? name)
+				 (if (pathname-absolute? pathname)
+				     (list "")
+				     '())
+				 (list name)))))
+	      #f
+	      #f)))
+
+(define (uri->pathname uri #!optional error?)
+  (let ((uri (->uri uri (and error? 'URI->PATHNAME)))
+	(defaults *default-pathname-defaults*)
+	(finish
+	 (lambda (device path keyword)
+	   (receive (directory name type)
+	       (if (pair? path)
+		   (let ((d (cons keyword (except-last-pair path)))
+			 (s (car (last-pair path))))
+		     (if (string-null? s)
+			 (values d #f #f)
+			 (let ((pn (parse-namestring s)))
+			   (values d
+				   (pathname-name pn)
+				   (pathname-type pn)))))
+		   (values (list keyword) #f #f))
+	     (make-pathname #f device directory name type #f)))))
+    (let ((scheme (uri-scheme uri))
+	  (path
+	   (map (lambda (x)
+		  (cond ((string=? x "*") 'WILD)
+			((string=? x "..") 'UP)
+			(else (utf8-string->string x))))
+		(uri-path uri)))
+	  (lose
+	   (lambda ()
+	     (if error? (error:bad-range-argument uri 'URI->PATHNAME))
+	     #f)))
+      (case scheme
+	((file)
+	 (if (and (pair? path)
+		  (string-null? (car path)))
+	     (let ((path (cdr path)))
+	       (receive (device path)
+		   (let ((device (pathname-device defaults)))
+		     (if (and (pair? path)
+			      (not (missing-component? device)))
+			 (values (car path) (cdr path))
+			 (values device path)))
+		 (if (pair? path)
+		     (finish device path 'ABSOLUTE)
+		     (lose))))
+	     (lose)))
+	((#f) (finish #f path 'RELATIVE))
+	(else (lose))))))
+
+(define (missing-component? x)
+  (or (not x)
+      (eq? x 'UNSPECIFIC)))
+
 ;;;; Pathname Syntax
 
 (define (parse-namestring namestring #!optional host defaults)
@@ -458,8 +540,7 @@ these rules:
   (operation/pathname->truename #f read-only #t)
   (operation/user-homedir-pathname #f read-only #t)
   (operation/init-file-pathname #f read-only #t)
-  (operation/pathname-simplify #f read-only #t)
-  (operation/end-of-line-string #f read-only #t))
+  (operation/pathname-simplify #f read-only #t))
 
 (define-structure (host (type vector)
 			(named ((ucode-primitive string->symbol)
@@ -508,45 +589,51 @@ these rules:
 	     local-host)))
     ((host-type/operation/init-file-pathname (host/type host)) host)))
 
-(define (system-library-pathname pathname)
-  (let ((try-directory
-	 (lambda (directory)
-	   (let ((pathname (merge-pathnames pathname directory)))
-	     (and (file-exists? pathname)
-		  pathname))))
-	(loser
-	 (lambda ()
-	   (system-library-pathname
-	    (->pathname
-	     (error:file-operation pathname
-				   "find"
-				   "file"
-				   "no such file in system library path"
-				   system-library-pathname
-				   (list pathname)))))))
-    (if (pathname-absolute? pathname)
-	(if (file-exists? pathname) pathname (loser))
-	(let loop ((directories library-directory-path))
-	  (if (null? directories)
-	      (loser)
-	      (or (try-directory (car directories))
-		  (loop (cdr directories))))))))
+(define (system-library-pathname pathname #!optional required?)
+  (if (if (default-object? required?) #t required?)
+      (or (%find-library-file pathname)
+	  (system-library-pathname
+	   (error:file-operation pathname
+				 "find"
+				 "file"
+				 "no such file in system library path"
+				 system-library-pathname
+				 (list pathname required?))
+	   required?))
+      (merge-pathnames pathname (%find-library-directory))))
+
+(define (system-library-directory-pathname #!optional pathname required?)
+  (if (if (default-object? pathname) #f pathname)
+      (let ((dir (%find-library-file pathname)))
+	(cond ((file-directory? dir)
+	       (pathname-as-directory dir))
+	      ((if (default-object? required?) #f required?)
+	       (system-library-directory-pathname
+		(error:file-operation
+		 pathname
+		 "find"
+		 "directory"
+		 "no such directory in system library path"
+		 system-library-directory-pathname
+		 (list pathname required?))
+		required?))
+	      (else #f)))
+      (%find-library-directory)))
+
+(define (%find-library-directory)
+  (pathname-simplify
+   (or (find-matching-item library-directory-path file-directory?)
+       (error "Can't find library directory."))))
+
+(define (%find-library-file pathname)
+  (let loop ((path library-directory-path))
+    (and (pair? path)
+	 (let ((p (merge-pathnames pathname (car path))))
+	   (if (file-exists? p)
+	       p
+	       (loop (cdr path)))))))
 
 (define library-directory-path)
-
-(define (system-library-directory-pathname pathname)
-  (if (not pathname)
-      (let ((pathname
-	     (list-search-positive library-directory-path file-directory?)))
-	(if (not pathname)
-	    (error "can't find system library directory"))
-	(pathname-as-directory pathname))
-      (let loop ((directories library-directory-path))
-	(and (not (null? directories))
-	     (let ((pathname (merge-pathnames pathname (car directories))))
-	       (if (file-directory? pathname)
-		   (pathname-as-directory pathname)
-		   (loop (cdr directories))))))))
 
 (define known-host-types
   '((0 UNIX)
@@ -596,7 +683,7 @@ these rules:
 	   (lambda arguments
 	     (error "Unimplemented host type:" name arguments))))
       (make-host-type index name fail fail fail fail fail fail fail fail fail
-		      fail fail fail fail fail))))
+		      fail fail fail fail))))
 
 (define (reset-package!)
   (let ((host-type (host-name->type microcode-id/operating-system))
@@ -619,4 +706,7 @@ these rules:
 
 (define (initialize-package!)
   (reset-package!)
-  (add-event-receiver! event:after-restore reset-package!))
\ No newline at end of file
+  (add-event-receiver! event:after-restore reset-package!))
+
+(define (initialize-parser-method!)
+  (define-bracketed-object-parser-method 'PATHNAME pathname-parser-method))
\ No newline at end of file
