@@ -1,9 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: record.scm,v 1.54 2005/09/08 19:12:37 cph Exp $
-
-Copyright 1989,1990,1991,1993,1994,1996 Massachusetts Institute of Technology
-Copyright 1997,2002,2003,2004,2005 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -33,8 +32,8 @@ USA.
 (declare (usual-integrations))
 
 (define-primitives
-  (%record? 1)
   (%record -1)
+  (%record? 1)
   (%record-length 1)
   (%record-ref 2)
   (%record-set! 3)
@@ -44,7 +43,8 @@ USA.
   (vector-cons 2))
 
 (define-integrable (%make-record length object)
-  (object-new-type (ucode-type record) (vector-cons length object)))
+  ((ucode-primitive object-set-type) (ucode-type record)
+				     (vector-cons length object)))
 
 (define-integrable (%record-tag record)
   (%record-ref record 0))
@@ -87,7 +87,11 @@ USA.
 	(let ((tag (cadr tags)))
 	  (cond ((record-type? (dispatch-tag-contents tag))
 		 (standard-unparser-method
-		  (%record-type-name (dispatch-tag-contents tag))
+		  (let ((name (%record-type-name (dispatch-tag-contents tag))))
+		    (if (and (string-prefix? "<" name)
+			     (string-suffix? ">" name))
+			(substring name 1 (fix:- (string-length name) 1))
+			name))
 		  #f))
 		((eq? tag record-type-type-tag)
 		 (standard-unparser-method 'RECORD-TYPE
@@ -130,7 +134,7 @@ USA.
 			  #!optional default-inits unparser-method)
   (let ((caller 'MAKE-RECORD-TYPE))
     (guarantee-list-of-unique-symbols field-names caller)
-    (let* ((names (list->vector field-names))
+    (let* ((names ((ucode-primitive list->vector) field-names))
 	   (n (vector-length names))
 	   (record-type
 	    (%record record-type-type-tag
@@ -189,7 +193,7 @@ USA.
   (guarantee-record-type record-type 'RECORD-TYPE-FIELD-NAMES)
   ;; Can't use VECTOR->LIST here because it isn't available at cold load.
   (let ((v (%record-type-field-names record-type)))
-    (subvector->list v 0 (vector-length v))))
+    ((ucode-primitive subvector->list) v 0 (vector-length v))))
 
 (define (record-type-default-inits record-type)
   (guarantee-record-type record-type 'RECORD-TYPE-DEFAULT-INITS)
@@ -223,14 +227,25 @@ USA.
 	(vector-set! v i (car values))))))
 
 (define (record-type-default-value record-type field-name)
+  (record-type-default-value-by-index
+   record-type
+   (record-type-field-index record-type field-name #t)))
+
+(define (record-type-default-value-by-index record-type field-name-index)
   ((vector-ref (%record-type-default-inits record-type)
-	       (fix:- (record-type-field-index record-type field-name #t) 1))))
+	       (fix:- field-name-index 1))))
 
 (define set-record-type-unparser-method!
   (named-lambda (set-record-type-unparser-method!/booting record-type method)
-    (set! deferred-unparser-methods
-	  (cons (cons record-type method) deferred-unparser-methods))
-    unspecific))
+    (let loop ((ms deferred-unparser-methods))
+      (if (pair? ms)
+	  (if (eq? (caar ms) record-type)
+	      (set-cdr! (car ms) method)
+	      (loop (cdr ms)))
+	  (begin
+	    (set! deferred-unparser-methods
+		  (cons (cons record-type method) deferred-unparser-methods))
+	    unspecific)))))
 
 (define deferred-unparser-methods '())
 
@@ -289,24 +304,23 @@ USA.
       (let ((tag (%record-type-dispatch-tag record-type))
 	    (n-fields (%record-type-n-fields record-type)))
 	(expand-cases tag n-fields 16
-	  (let ((length (fix:+ 1 n-fields)))
+	  (let ((reclen (fix:+ 1 n-fields)))
 	    (letrec
 		((constructor
 		  (lambda field-values
-		    (let ((record (%make-record length #f))
+		    (let ((record (%make-record reclen #f))
 			  (lose
 			   (lambda ()
 			     (error:wrong-number-of-arguments constructor
 							      n-fields
 							      field-values))))
 		      (%record-set! record 0 tag)
-		      (let loop ((i 1) (values field-values))
-			(if (fix:< i length)
-			    (begin
-			      (if (not (pair? values)) (lose))
-			      (%record-set! record i (car values))
-			      (loop (cdr values) (fix:+ i 1)))
-			    (if (not (null? values)) (lose))))
+		      (do ((i 1 (fix:+ i 1))
+			   (vals field-values (cdr vals)))
+			  ((not (fix:< i reclen))
+			   (if (not (null? vals)) (lose)))
+			(if (not (pair? vals)) (lose))
+			(%record-set! record i (car vals)))
 		      record))))
 	      constructor)))))))
 
@@ -369,6 +383,7 @@ USA.
 	  (let ((n (%record-type-length record-type)))
 	    (let ((record (%make-record n #f))
 		  (seen? (vector-cons n #f)))
+	      (%record-set! record 0 (%record-type-dispatch-tag record-type))
 	      (do ((kl keyword-list (cddr kl)))
 		  ((not (and (pair? kl)
 			     (symbol? (car kl))
@@ -453,16 +468,9 @@ USA.
 					error?))))))
 
 (define (->type-name object)
-  (let* ((string
-	  (cond ((string? object) object)
-		((symbol? object) (symbol-name object))
-		(else (error:wrong-type-argument object "type name" #f))))
-	 (n (string-length string)))
-    (if (and (fix:> n 2)
-	     (char=? (string-ref string 0) #\<)
-	     (char=? (string-ref string (fix:- n 1)) #\>))
-	(substring string 1 (fix:- n 1))
-	string)))
+  (cond ((string? object) object)
+	((symbol? object) (symbol-name object))
+	(else (error:wrong-type-argument object "type name" #f))))
 
 (define (list-of-unique-symbols? object)
   (and (list-of-type? object symbol?)
@@ -537,8 +545,12 @@ USA.
 	      (structure-type/field-name-index type field-name)))
 
 (define-integrable (structure-type/default-init type field-name)
-  (vector-ref (structure-type/default-inits type)
-	      (structure-type/field-name-index type field-name)))
+  (structure-type/default-init-by-index
+   type
+   (structure-type/field-name-index type field-name)))
+
+(define-integrable (structure-type/default-init-by-index type field-name-index)
+  (vector-ref (structure-type/default-inits type) field-name-index))
 
 (define (structure-type/field-name-index type field-name)
   (let ((names (structure-type/field-names type)))
@@ -589,6 +601,9 @@ USA.
 (define (define-structure/default-value type field-name)
   ((structure-type/default-init type field-name)))
 
+(define (define-structure/default-value-by-index type field-name-index)
+  ((structure-type/default-init-by-index type field-name-index)))
+
 (define (define-structure/keyword-constructor type)
   (let ((names (structure-type/field-names type))
 	(indexes (structure-type/field-indexes type))
