@@ -1,9 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: starbase.scm,v 1.20 2003/02/14 18:28:34 cph Exp $
-
-Copyright 1989,1990,1991,1992,1993,1994 Massachusetts Institute of Technology
-Copyright 1995,2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +18,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -136,7 +135,7 @@ USA.
 (define-accessors-and-mutators text-rotation)
 
 (define (operation/available?)
-  (implemented-primitive-procedure? starbase-open-device))
+  (implemented-primitive-procedure? (ucode-primitive starbase-open-device 2)))
 
 (define (operation/open descriptor->device device-name driver-name)
   (let ((identifier (starbase-open-device device-name driver-name)))
