@@ -1,10 +1,8 @@
 #| -*-Scheme-*-
 
-$Id: turtle.scm,v 1.43 2008/01/30 20:02:42 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -129,10 +127,8 @@ USA.
    (with-pointer p
      (alt (map make-rdf-bnode
 	       (seq "_:" parse:name))
-	  ;; This notation should probably accept whitespace between the
-	  ;; brackets, but the spec is written like this:
 	  (encapsulate (lambda (v) v (make-rdf-bnode))
-	    "[]")
+	    (seq "[" parse:ws* "]"))
 	  (map (lambda (pols) (cons 'BLANK-NODE pols))
 	       (seq "["
 		    parse:ws*
@@ -282,7 +278,7 @@ USA.
 		  char-set:turtle-digit))
 
 (define alphabet:name-start-char
-  (code-points->alphabet
+  (scalar-values->alphabet
    '((#x0041 . #x005A)
      #x005F
      (#x0061 . #x007A)
@@ -301,7 +297,7 @@ USA.
 
 (define alphabet:name-char
   (alphabet+ alphabet:name-start-char
-	     (code-points->alphabet
+	     (scalar-values->alphabet
 	      '(#x002D
 		(#x0030 . #x0039)
 		#x00B7
@@ -312,7 +308,7 @@ USA.
   (alphabet- alphabet:name-start-char (alphabet #\_)))
 
 (define alphabet:character
-  (code-points->alphabet '((#x20 . #x5B) (#x5D . #x10FFFF))))
+  (scalar-values->alphabet '((#x20 . #x5B) (#x5D . #x10FFFF))))
 
 (define alphabet:ucharacter
   (alphabet- alphabet:character (alphabet #\>)))
@@ -328,7 +324,7 @@ USA.
 (define (delimited-region-parser name start-delim end-delim
 				 alphabet parse-escapes)
   (lambda (buffer)
-    (let ((output (open-output-string))
+    (let ((output (open-utf8-output-string))
 	  (start (get-parser-buffer-pointer buffer)))
 
       (define (read-head)
@@ -373,7 +369,6 @@ USA.
       (define (finish)
 	(vector (get-output-string output)))
 
-      (port/set-coding output 'UTF-8)
       (and (match-parser-buffer-string buffer start-delim)
 	   (read-head)))))
 
@@ -511,11 +506,10 @@ USA.
 	       pols))
 
 (define (post-process-resource resource prefixes base-uri)
-  (cond ((or (absolute-uri? resource)
-	     (rdf-bnode? resource)
+  (cond ((or (rdf-bnode? resource)
 	     (rdf-literal? resource))
 	 (values resource '()))
-	((relative-uri? resource)
+	((uri? resource)
 	 (values (merge-uris resource base-uri) '()))
 	((pair? resource)
 	 (case (car resource)
@@ -593,7 +587,7 @@ USA.
     (newline port)))
 
 (define (write-prefixes graph port)
-  (let ((table (make-eq-hash-table)))
+  (let ((table (make-strong-eq-hash-table)))
 
     (define (check-graph g)
       (for-each check-triple (rdf-graph-triples g)))
@@ -771,18 +765,18 @@ USA.
 		    (else #f))))
 	((rdf-bnode? o)
 	 (and (not (inline-bnode o))
-	      (call-with-output-string
+	      (call-with-utf8-output-string
 		(lambda (port)
 		  (write-rdf/nt-bnode o port)))))
 	((uri? o)
-	 (call-with-output-string
+	 (call-with-utf8-output-string
 	   (lambda (port*)
 	     (write-uri o (port/rdf-prefix-registry port) port*))))
 	((rdf-graph? o)
 	 (and (null? (rdf-graph-triples o))
 	      "{}"))
 	((rdf-literal? o)
-	 (call-with-output-string
+	 (call-with-utf8-output-string
 	   (lambda (port)
 	     (write-rdf/turtle-literal o port))))
 	(else
@@ -917,8 +911,7 @@ USA.
 
 (define (write-literal-text text port)
   (if (string-find-next-char text #\newline)
-      (let ((tport (open-input-string text)))
-	(port/set-coding tport 'UTF-8)
+      (let ((tport (open-utf8-input-string text)))
 	(write-string "\"\"\"" port)
 	(let loop ()
 	  (let ((char (read-char tport)))
